PrecacheModel("models/Weapons/w_grenade.mdl"); //todo do we still need this?

::ProjectileInfo <- class
{
    projectiles = {};
    damageUnspecified = null;
    damageDirect = null;
    blastDamage = null;
    blastRadius = null;
    damageFalloff = null;

    damageInflictor = null;
    damageType = null;

    model = null;
    speed = null;
    offset = null;
    rotOffset = null;
    gravity = null;

    shooter = null;
    weapon = null;
    rotation = null;
    position = null;

    projectile = null;
    projectileEntIndex = null;
    size = null;

    shouldHitTeammates = null;
    onHit = null;

    constructor(projDef, projParams)
    {
        projectiles[this] <- 1;
        damageUnspecified = safeget(projDef, "damage", null);
        damageDirect = safeget(projDef, "damageDirect", damageUnspecified);
        blastDamage = safeget(projDef, "blastDamage", null);
        blastRadius = safeget(projDef, "blastRadius", null);
        damageFalloff = safeget(projDef, "damageFalloff", 0);

        shouldHitTeammates = safeget(projDef, "shouldHitTeammates", 0);
        onHit = safeget(projParams, "onHit", null);

        damageInflictor = safeget(projDef, "damageInflictor", null)
        damageType = safeget(projDef, "damageType", 0);

        model = safeget(projDef, "model", "models/empty.mdl");
        speed = projDef.speed;
        offset = safeget(projDef, "offset", Vector());
        rotOffset = safeget(projDef, "rotOffset", QAngle());
        gravity = safeget(projDef, "gravity", null);

        shooter = safeget(projParams, "shooter", null);
        weapon = safeget(projParams, "weapon", null);
        rotation = safeget(projParams, "rotation", shooter.EyeAngles());
        position = safeget(projParams, "position", shooter.EyePosition());

        if (offset)
            position = position + RotatePosition(Vector(), rotation, offset);
        if (rotOffset)
            rotation = rotation + rotOffset;

        local velocity = safeget(projParams, "velocity", rotation.Forward() * speed);
        local team = safeget(projParams, "team", shooter ? shooter.GetTeam() : 0);

        projectile = CreateByClassname("tf_projectile_throwable");
        projectile.SetAbsOrigin(position);
        projectile.SetAbsAngles(rotation);
        projectile.SetPhysVelocity(velocity);
        projectile.DispatchSpawn();
        projectile.SetModelSimple(model);
        projectile.SetTeam(team);
        projectileEntIndex = projectile.entindex();

        if (gravity)
        {
            projectile.SetMoveType(MOVETYPE_FLYGRAVITY, MOVECOLLIDE_DEFAULT);
            projectile.SetGravity(gravity);
            projectile.SetAbsVelocity(velocity);
            //projectile.SetAngularVelocity(RandomFloat(-1000, 1000), RandomFloat(-1000, 1000), RandomFloat(-1000, 1000));
        }
        else
            projectile.SetPhysVelocity(velocity);

        size = safeget(projDef, "size", null);
        if (!size)
        {
            local delta = projectile.GetBoundingMaxs() - projectile.GetBoundingMins();
            size = max(delta.x, delta.y);
        }
        size = Vector(size, size, size);
        projectile.SetSize(size * -0.5, size * 0.5);

        AddTimer(-1, TickProjectileCollision)
    }

    function TickProjectileCollision()
    {
        printl("TickProjectileCollision")
        if (projectile.IsValid() && !GetPropBool(projectile, "m_bTouched"))
            return;

        delete projectiles[this];
        projectile = EntIndexToHScript(projectileEntIndex);
        if (!projectile)
            return TIMER_DELETE;

        local projectilePos = projectile.GetCenter();
        projectile.SetAbsOrigin(Vector(-99999, -99999, -99999));

        //If direct hits only
        local directHitTarget = null;
        printl("damageDirect ="+damageDirect)
        if (damageDirect)
        {
            local traceHull = {
                start = projectilePos,
                end = projectilePos,
                hullmin = size * -0.5,
                hullmax = size * 0.5
            }
            TraceHull(traceHull);
            directHitTarget = traceHull.hit ? traceHull.enthit : null;
            if (directHitTarget)
            {
                printl("shouldHitTeammates")
                if (!shouldHitTeammates || (directHitTarget.GetTeam() == shooter.GetTeam()))
                {
                    if (onHit && onHit(directHitTarget))
                        directHitTarget.TakeDamageEx(
                            damageInflictor,
                            shooter,
                            shooter.GetActiveWeapon(),
                            Vector(),
                            Vector(),
                            damageDirect,
                            damageType);
                    }
            }
        }
        if (blastDamage)
        {
            local explosion = SpawnEntityFromTable("env_explosion",
            {
                iMagnitude = blastDamage,
                iRadiusOverride = blastRadius,
                origin = projectilePos,
                teamnum = shooter.GetTeam(),
                spawnflags = 6740
            });
            explosion.SetOwner(shooter);
            SetPropEntity(explosion, "m_hEntityIgnore", directHitTarget);
            explosion.AcceptInput("Explode", null, shooter, shooter);
        }
        KillIfValid(projectile);
        return TIMER_DELETE;
    }
}

::ShootProjectileBase <- function(projDef, projParams)
{
    return ProjectileInfo(projDef, projParams).projectile;
}

OnGameEvent("OnTakeDamage", 0, function(victim, params)
{
    if (params.inflictor && params.inflictor.GetClassname() == "env_explosion")
    {
        //params.damage_type = DMG_SLOWBURN; //todo
        params.attacker = params.inflictor.GetOwner();
    }
});

::ShootProjectileTest <- function(projDef, projParams)
{
    local projectile = ShootProjectileBase(projDef, projParams);
    projectile.ValidateScriptScope();
    projectile.GetScriptScope().shooter <- player;
    projectile.GetScriptScope().team <- projectile.GetTeam();
    projectile.SetOwner(null);
    SetPropEntity(projectile, "m_hThrower", null);
    projectile.SetTeam(0);

    return projectile;
}

::ShootProjectileTeammateCollision <- function(projDef, projParams)
{
    local projectile = ShootProjectileBase(projDef, projParams);
    projectile.ValidateScriptScope();
    projectile.GetScriptScope().shooter <- player;
    projectile.GetScriptScope().team <- projectile.GetTeam();
    projectile.SetOwner(null);
    SetPropEntity(projectile, "m_hThrower", null);
    projectile.SetTeam(0);

    OnGameEvent("OnTakeDamage", -99, function(victim, params)
    {
        if (params.inflictor != self)
            return;
        self.SetOwner(shooter);
        SetPropEntity(self, "m_hThrower", shooter);
        projectile.SetTeam(team);
        params.attacker = shooter;
        if (victim.GetTeam() == team)
        {
            if ("onTeammateHit" in projDef)
                projDef.onTeammateHit.call(this, victim);
            params.early_out = true;
            params.damage = 0;
            self.Kill();
            return EVENT_DELETE;
        }
        else if ("onEnemyHit" in projDef)
            projDef.onEnemyHit.call(this, victim);
    }, projectile);

    return projectile;
}