::AbilityOffhandCoin <- class extends BaseAbility
{
    //Custom character definition
    worldmodelPath = "models/vip_julius/weapons/w_coin.mdl";
    viewmodelPath = "models/vip_julius/weapons/v_coin.mdl";
    buffs = [
        TF_COND_OFFENSEBUFF,
        TF_COND_HALLOWEEN_QUICK_HEAL,
        TF_COND_SPEED_BOOST
    ]

    maxCooldown = 1;

    rechargeLength = 2;
    maxCharges = 3;
    currentCharges = 3;

    buffTimer = function(player, buff, vip, endTS)
    {
        if (Time() > endTS)
            return TIMER_DELETE;
        if (!player.InCond(TF_COND_STEALTHED) && !player.InCond(TF_COND_DISGUISED))
            player.AddCondEx(buff, 0.15, vip);
    }

    projDef = {
        speed = 1600,
        damage = 50,
        size = 20,
        gravity = 0.1,
        shouldHitTeammates = 1,
        rotOffset = QAngle(0, 0, 0),
        offset = Vector(50, 0, 0),
        onTeammateHit = function(teammate)
        {
            local coinType = projectile.GetScriptScope().coinType;
            SetPropVector(teammate, "m_Local.m_vecPunchAngle", Vector())
            if (coinType == 1)
                teammate.AddCondEx(AbilityOffhandCoin.buffs[coinType], 3, projectile.GetOwner());
            else
                AddTimer(0.1, AbilityOffhandCoin.buffTimer, teammate, AbilityOffhandCoin.buffs[coinType], projectile.GetOwner(), Time() + 5, teammate);
        }
    };

    //Dynamic data
    offhandVMEnt = null;
    coinType = 0;

    function Precache()
    {
        projDef.model <- worldmodelPath,
        PrecacheModel(worldmodelPath);
        PrecacheModel(viewmodelPath);
    }

    function ApplyAbility()
    {
        base.ApplyAbility();

        offhandVMEnt = player.CreateViewModel(viewmodelPath);
        offhandVMEnt.ResetSequence(offhandVMEnt.LookupSequence("coin_idle"));
        offhandVMEnt.SetCycle(0);
        offhandVMEnt.SetPlaybackRate(1);
        AddTimer(-1, TickAbility);
    }

    function TickAbility()
    {
        if (CanPerform())
            Perform();
    }

    function CheckInput()
    {
        return GetPropInt(player, "m_nButtons") & IN_ATTACK2;
    }

    function Perform()
    {
        base.Perform();
        ConsumeCharge();
        ConsumeCooldown();

        /*local projectile = ShootProjectileTeammateCollision(projDef, {
            shooter = player,
            weapon = player.GetActiveWeapon()
        });*/
        /*local projectile = ShootProjectileTeammateCollision(projDef, {
            shooter = player,
            weapon = player.GetActiveWeapon(),
            onHit = function(target)
            {
                true;
            }
        })*/
        local projectile = ProjectileInfoTest().Build({
            shooter = player,
            weapon = player.GetActiveWeapon()
        })
        projectile.ValidateScriptScope();
        projectile.GetScriptScope().coinType <- coinType;
        projectile.SetSkin(coinType);
        if (coinType == 0)
        {
            charScript.EmitSoundBankSimple("throw_krits_sfx");
            charScript.EmitSoundBankSimple("projectile_krits_sfx", { entity = projectile });
        }
        else if (coinType == 1)
        {
            charScript.EmitSoundBankSimple("throw_healing_sfx");
            charScript.EmitSoundBankSimple("projectile_healing_sfx", { entity = projectile });
        }
        else if (coinType == 2)
        {
            charScript.EmitSoundBankSimple("throw_speed_sfx");
            charScript.EmitSoundBankSimple("projectile_speed_sfx", { entity = projectile });
        }
        offhandVMEnt.ResetSequence(offhandVMEnt.LookupSequence("coin_throw"));
        offhandVMEnt.SetCycle(0);
        RunWithDelay(1, function()
        {
            offhandVMEnt.ResetSequence(offhandVMEnt.LookupSequence("coin_idle"));
            offhandVMEnt.SetCycle(0);
        });
    }

    function SwapCoin(coinType)
    {
        this.coinType = coinType;
        offhandVMEnt.SetBodygroup(0, coinType);
        offhandVMEnt.ResetSequence(offhandVMEnt.LookupSequence("coin_draw"));
        offhandVMEnt.SetCycle(0);
        RunWithDelay(0.2, function()
        {
            offhandVMEnt.ResetSequence(offhandVMEnt.LookupSequence("coin_idle"));
            offhandVMEnt.SetCycle(0);
        });
    }
}