::VIPMercCharacter <- class extends CustomCharacter
{
    setupLinesPlayedInTotal = [0];

    //Definitions
    deleteItems = false;
    mute = false;
    keepAfterDeath = false;
    keepAfterClassChange = false;
    cleanAttachments = false;

    function ApplyCharacter()
    {
        base.ApplyCharacter();

        AddTimer(RandomFloat(0.49, 0.51), TickEnemyLocationVO);
        RemovePyrovision();
    }

    function RemovePyrovision()
    {
        foreach(item in player.GetWeaponsAndCosmetics())
	        item.AddAttribute("vision opt in flags", 0, -1);
    }

    function OnRegenerate(fakeRegenTrigger)
    {
        return 0;
    }

    //======================================
    //Enemy location lines
    //======================================

    tickInverval = 0.5;
    sharedPlayInterval = 20;
    sharedPlayPool = 3;
    lastTimeBossWasInSightline = Time() + 16 + 10;
    lastTimeLookedAtBoss = Time() + 16 + 10;
    lastTimePlayedLine = {};

    function TickEnemyLocationVO()
    {
        local boss = FindVIP(player.GetEnemyTeam());
        if (!boss)
            return;
        local time = Time();

        local myEyePos = player.EyePosition();
        local bossCenter = boss.GetCenter();

        //Checking light of sight between this player and Boss
        if (TraceLine(myEyePos, bossCenter, null) < 0.99)
            return;

        local deltaVector = bossCenter - myEyePos;
        local playerEyes = player.EyeAngles().Forward();

        if (!ShouldPlayVoiceLine(time))
        {
            lastTimeBossWasInSightline = time;
            deltaVector.Norm();
            if (playerEyes.Dot(deltaVector) > 0.3)
                lastTimeLookedAtBoss = lastTimeBossWasInSightline;
            return;
        }

        local deltaVectorXY = Vector(deltaVector.x, deltaVector.y, 0);
        local playerEyesXY = Vector(playerEyes.x, playerEyes.y, 0);

        deltaVector.Norm();
        playerEyes.Norm();
        deltaVectorXY.Norm();
        playerEyesXY.Norm();

        local dotXY = playerEyesXY.Dot(deltaVectorXY);
        local deltaZ = boss.IsOnGround() ? 0 : deltaVector.z;
        local timeWithNoSightline = time - lastTimeBossWasInSightline;
        local timeNotLookedAtBoss = time - lastTimeLookedAtBoss;

        /*if (deltaZ > 0.3 && timeNotLookedAtBoss > 4 && deltaVectorXY.Length() < 1300)
        {
        }
        else if (dotXY < -0.3 && timeNotLookedAtBoss > 7 && deltaVector.Length() < 1300)
        {
        }
        else*/ if (dotXY > 0.2 && timeWithNoSightline > 7)
        {
            lastTimePlayedLine[player] <- time;
            EmitVoiceLine("contact", {
                filter_type = RECIPIENT_FILTER_TEAM,
                filter_param = player.GetTeam()
            });
        }

        lastTimeBossWasInSightline = time;
        if (dotXY > 0.3)
            lastTimeLookedAtBoss = lastTimeBossWasInSightline;
    }

    function ShouldPlayVoiceLine(time)
    {
        if (lastTimePlayedLine.len() > 200) //Easy anti-memory leak fix
            lastTimePlayedLine = {};
        local playSlotsOccupied = 0;
        foreach (teammate, lastPlayTime in lastTimePlayedLine)
            if (time - lastPlayTime < sharedPlayInterval
                && (++playSlotsOccupied >= sharedPlayPool || player == teammate || time - lastPlayTime < 3))
                    return false;
        return true;
    }
}

Include("vip/characters/merc_changes/scout.nut");
Include("vip/characters/merc_changes/soldier.nut");
//Include("vip/characters/merc_changes/pyro.nut");
Include("vip/characters/merc_changes/demo.nut");
Include("vip/characters/merc_changes/heavy.nut");
Include("vip/characters/merc_changes/engineer.nut");
Include("vip/characters/merc_changes/medic.nut");
//Include("vip/characters/merc_changes/sniper.nut");
//Include("vip/characters/merc_changes/spy.nut");

::VIP_CLASS_CHARACTERS <- [
    null, //GENERIC
    VIPScout, //SCOUT
    null, //SNIPER
    VIPSoldier, //SOLDIER
    VIPDemo, //DEMO
    VIPMedic, //MEDIC
    VIPHeavy, //HEAVY
    null, //PYRO
    null, //SPY
    VIPEngineer, //ENGINEER
    null  //CIVLIAN (not that one)
];