PrecacheScriptSound("VIPMobster.Typewriter.Single");
PrecacheScriptSound("VIPMobster.Typewriter.NoAmmo");
PrecacheScriptSound("VIPMobster.Typewriter.LowAmmo");
PrecacheSound("weapons/pistol_shoot_crit.wav");
PrecacheModel("models/vip_mobster/weapons/w_moneybag_open.mdl");

class ::MobsterTypeWriter extends CustomWeaponBase
{
    baseclass = "tf_weapon_handgun_scout_primary";
    baseid = 220;
    viewModels = [
        "models/vip_mobster/weapons/v_typewriter.mdl",
        "models/vip_mobster/weapons/c_mobster_arms.mdl"
    ];
    worldModels = [
        "models/vip_mobster/weapons/w_typewriter.mdl"
    ];
    clip = 40;
    maxAmmo = 200;
    mute = true;

    playerInTaunt = false;
    needToPlayEmptyClipSFX = 0;
    isFiringCrits = false;
    nextShoveTime = 0;

    function OnEquip()
    {
        weapon.AddAttribute("bullets per shot bonus", 0.75, -1);
        weapon.AddAttribute("fire rate bonus", 0.34, -1);
        weapon.AddAttribute("clip size bonus", 10, -1);
        weapon.AddAttribute("damage bonus HIDDEN", 0.45, -1);
        weapon.AddAttribute("dmg falloff decreased", 0.35, -1);
        weapon.AddAttribute("spread penalty", 2, -1);
        weapon.AddAttribute("Reload time increased", 1.33, -1);
        weapon.AddAttribute("dmg bonus vs buildings", 1.25, -1);

        AddTimer(-1, ProcessTypewriterTaunt);
        //AddTimer(-1, ProcessShove);
        AddTimer(0.05, ProcessEmptyAmmoSFX);
        OnGameEvent("OnWorldHit", function(params)
        {
            if (weapon == params.weapon)
                isFiringCrits = params.damage_type & DMG_CRIT;
        })
    }

    function OnSwitchTo()
    {
        player.AddCustomAttribute("gesture speed increase", 0.752, -1);
    }

    function OnFire()
    {
        player.ViewPunch(QAngle(-0.2, 0, 0));
        local restore = GetPropVector(player, "m_vecViewOffset");
        //SetPropVector(player, "m_vecViewOffset", Vector(999999, 999999, 999999));
        if (!isFiringCrits)
            EmitSoundEx({
                sound_name = "VIPMobster.Typewriter.Single",
                //filter_type = RECIPIENT_FILTER_PAS_ATTENUATION,
                entity = player,
                speaker_entity = player
            });
        else
            EmitSoundEx({
                sound_name = "weapons/pistol_shoot_crit.wav",
                //channel = 6,
                //volume = 1,
                //filter_type = RECIPIENT_FILTER_PAS_ATTENUATION,
                entity = player,
                speaker_entity = player
            });
        if (weapon.Clip1() <= 15)
            EmitSoundEx({
                sound_name = "VIPMobster.Typewriter.LowAmmo",
                //channel = 6,
                //filter_type = RECIPIENT_FILTER_PAS_ATTENUATION,
                entity = player,
                speaker_entity = player
            });
        //SetPropVector(player, "m_vecViewOffset", restore);
    }

    function OnDamage(victim, params)
    {
        isFiringCrits = params.damage_type & DMG_CRIT;

        //Making the <s>Shortstop</s> Typewriter Shove do 65 damage instead of 1.
        if (params.damage_type & DMG_MELEE)
        {
            params.damage = 65;
            params.damage_type = params.damage_type | DMG_PREVENT_PHYSICS_FORCE;
        }
    }

    function ProcessShove()
    {
        if (!(GetPropInt(player, "m_nButtons") & IN_ATTACK2))
            return;
        local time = Time();
        //if (time < nextShoveTime)
        //    return;
            printl("a")
        nextShoveTime = time + 1.5;
        SetPropFloat(weapon, "m_flNextAttack", Time()+10);
        SetPropFloat(weapon, "m_flNextPrimaryAttack", 0);
        SetPropFloat(weapon, "m_flNextSecondaryAttack", 0);
    }

    function ProcessTypewriterTaunt()
    {
        if (player.IsTaunting())
        {
            if (playerInTaunt)
                return;
            playerInTaunt = true;
            player.StopTaunt(true);
            player.RemoveCond(TF_COND_TAUNTING);
            weapon.KeyValueFromString("classname", "tf_weapon_scattergun");
            //SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 13);
            player.HandleTauntCommand(0);
            weapon.KeyValueFromString("classname", "tf_weapon_handgun_scout_primary");
            //SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 220);
        }
        else
            playerInTaunt = false;
    }

    function ProcessEmptyAmmoSFX()
    {
        if (weapon.Clip1() > 0)
        {
            needToPlayEmptyClipSFX = true;
            return;
        }
        if (!needToPlayEmptyClipSFX)
            return;
        needToPlayEmptyClipSFX = false;

        EmitSoundEx({
            sound_name = "VIPMobster.Typewriter.NoAmmo",
            entity = player,
            channel = 6,
            sound_level = 90
        });
    }
};

class ::MobsterLeadPipe extends CustomWeaponBase
{
    baseclass = "tf_weapon_shovel";
    baseid = 450;
    viewModels = [
        "models/vip_mobster/weapons/v_leadpipe.mdl",
        "models/vip_mobster/weapons/c_mobster_arms.mdl"
    ];
    worldModels = [
        "models/vip_mobster/weapons/w_leadpipe.mdl"
    ];

    modelState = 0;

    function OnEquip()
    {
        weapon.AddAttribute("dmg penalty vs players", 1, -1);
        weapon.AddAttribute("single wep deploy time increased", 1, -1);
    }

    function OnSwitchTo()
    {
        player.AddCustomAttribute("gesture speed increase", 1.27, -1);
    }

    function OnDamage(victim, params)
    {
        local newValue = params.damage_type & DMG_CRIT || params.damage >= victim.GetHealth() ? 2 : 1;
        if (newValue > modelState)
        {
            hViewModels[0].SetBodygroup(0, newValue);
            modelState = newValue;
        }
    }
};

class ::MobsterMoneyBag extends CustomWeaponBase
{
    baseclass = "tf_weapon_cleaver";
    baseid = 812;
    viewModels = [
        "models/vip_mobster/weapons/v_moneybag.mdl",
        "models/vip_mobster/weapons/c_mobster_arms.mdl"
    ];
    worldModels = [
        "models/vip_mobster/weapons/w_moneybag_closed.mdl"
    ];

    function OnEquip()
    {
        SetPropIntArray(player, "m_iAmmo", 1, weapon.GetPrimaryAmmoType());
        weapon.AddAttribute("effect bar recharge rate increased", 6, -1);
        weapon.AddAttribute("killstreak_tier", 1, -1);
        weapon.AddAttribute("gesture speed increase", 1, -1);
    }

    function OnSwitchTo()
    {
        player.AddCustomAttribute("gesture speed increase", 1.0, -1);
    }

    function OnFire()
    {
        AddTimer(-1, ProcessKillingCleaverProjectile, Time() + 3);

        RunWithDelay(0.2, function() {
            MoneyBagProjectile(player);
        });
    }

    function ProcessKillingCleaverProjectile(endTime)
    {
        if (Time() > endTime)
            return TIMER_DELETE;
        for (local projectile = null; projectile = FindByClassnameWithin(projectile, "tf_projectile_cleaver", player.GetCenter(), 128);)
        {
            if (GetPropEntity(projectile, "m_hThrower") == player)
            {
                projectile.Kill();
                return TIMER_DELETE;
            }
        }
    }
};

class ::MoneyBagFountain
{
    model = "models/vip_mobster/weapons/w_moneybag_open.mdl";
    deploySound = "VIPMobster.MoneyBag.Deploy";
    particle = "utaunt_cash_confetti";
    duration = 5.0;

    hMoneyBagOwner = null;
    hPropDynamic = null;

    affectedPlayers = null; //{}

    constructor(hMoneyBagOwner, origin)
    {
        PrecacheScriptSound(deploySound);
        this.hMoneyBagOwner = hMoneyBagOwner;

        AddTimer(0.2, TickMoneyBagFountain);

        affectedPlayers = {};

        local trace = {
            start = origin,
            end = origin - Vector(0, 0, 100),
            mask = CONTENTS_HITBOX|CONTENTS_SOLID|CONTENTS_MOVEABLE|CONTENTS_PLAYERCLIP|CONTENTS_WINDOW|CONTENTS_GRATE|CONTENTS_MONSTER,
            //ignore = hPhysProp
        };
        TraceLineEx(trace);
        local endpos = safeget(trace, "endpos", null);
        if (endpos)
            origin = endpos;

        hPropDynamic = SpawnEntityFromTable("prop_dynamic",
        {
            origin = origin,
            model = model,
            teamnum = hMoneyBagOwner.GetTeam()
        });
        hPropDynamic.AcceptInput("DisableShadow", null, null, null);
        hPropDynamic.ValidateScriptScope();
        hPropDynamic.GetScriptScope().script <- this;
        hPropDynamic.SetOwner(hMoneyBagOwner);
        EntFireByHandle(hPropDynamic, "Kill", null, duration, null, null);

        EmitSoundEx({
            sound_name = deploySound,
            entity = hPropDynamic,
            volume = 1,
            sound_level = 90
        });

        //Spawn money particle placeholders
        for (local x = -175; x <= 200; x += 87)
            for (local y = -175; y <= 200; y += 87)
                EntFireByHandle(SpawnEntityFromTable("info_particle_system",
                {
                    origin = hPropDynamic.GetOrigin() + Vector(x, y, 0),
                    effect_name = particle,
                    start_active = 1
                }), "SetParent", "!activator", -1, hPropDynamic, hPropDynamic);
    }

    function TickMoneyBagFountain()
    {
        if (!hPropDynamic || !hPropDynamic.IsValid())
            return TIMER_DELETE;
        local myPos = hPropDynamic.GetOrigin();
        foreach (nearbyPlayer in GetAlivePlayers(hPropDynamic.GetTeam()))
            if ((myPos - nearbyPlayer.GetOrigin()).Length() < 325)
            {
                if (nearbyPlayer in affectedPlayers)
                    affectedPlayers[nearbyPlayer].expireTime = Time() + 1;
                else
                    affectedPlayers[nearbyPlayer] <- MoneyBagEffect(hMoneyBagOwner, nearbyPlayer);
            }
    }
}

class ::MoneyBagEffect
{
    hEffectCarrier = null;
    hMoneyBagOwner = null;

    expireTime = null;

    constructor(hMoneyBagOwner, hEffectCarrier)
    {
        this.hEffectCarrier = hEffectCarrier;
        this.hMoneyBagOwner = hMoneyBagOwner;
        expireTime = Time() + 1;

        AddTimer(0.2, Think);
        OnGameEvent("OnTakeDamage", OnTakeDamage)
        OnGameEvent("player_hurt", 0, OnPlayerHurt)
    }

    function Think()
    {
        if (!hEffectCarrier || Time() > expireTime || !hEffectCarrier.IsValid()) //todo
            return TIMER_DELETE;

        hEffectCarrier.Heal(hEffectCarrier.GetMaxHealth() * 0.03, false);
    }

    function OnTakeDamage(victim, params)
    {
        if (victim == hEffectCarrier)
        {
            PlayDamageResistSound(victim, 100.0, 85.0);
            params.damage *= 0.85;
        }
        else if (params.attacker == hEffectCarrier)
            params.damage *= 1.15;
    }

    //This is to grant the assist to the VIP. It doesn't actually make people take increased damage.
    function OnPlayerHurt(victim, params)
    {
        if (victim.GetHealth() > 0)
            return;

        local attacker = GetPlayerFromUserID(params.attacker);
        if (!attacker || hEffectCarrier != attacker || attacker == hMoneyBagOwner)
            return;

        victim.StunPlayer(0, 0, 2, hMoneyBagOwner);
        victim.StopSound("TFPlayer.StunImpact");
        attacker.StopSound("TFPlayer.StunImpact");
    }
}

class ::MoneyBagProjectile
{
    model = "models/vip_mobster/weapons/w_moneybag_closed.mdl";
    moneyBagFountainClass = MoneyBagFountain;

    hPhysProp = null;
    hAirblastDetector = null;
    hOwner = null;

    constructor(hOwner)
    {
        this.hOwner = hOwner;

        local forwardVelocity = CalcThrowVelocity();

        hPhysProp = SpawnEntityFromTable("prop_physics_override", {
            model = model,
            origin = hOwner.EyePosition(),
            spawnflags = 4,
            massscale = 5
        });
        hPhysProp.SetModelScale(1.4, -1);
        hPhysProp.SetPhysVelocity(forwardVelocity);
        hPhysProp.SetPhysAngularVelocity(Vector(-550, -550, 050));
        hPhysProp.ValidateScriptScope();
        hPhysProp.GetScriptScope().script <- this;
        hPhysProp.AcceptInput("DisableShadow", null, null, null);

        RunWithDelay(0.1, function()
        {
            if (hPhysProp && hPhysProp.IsValid())
                hPhysProp.SetPhysVelocity(forwardVelocity);
        });

        hAirblastDetector = CreateByClassname("tf_projectile_cleaver");
        hAirblastDetector.SetAbsOrigin(hPhysProp.GetCenter());
        SetPropInt(hAirblastDetector, "m_fEffects", 129);
        SetPropInt(hAirblastDetector, "m_nNextThinkTick", 0x7FFFFFFF);
        hAirblastDetector.SetSolid(0);
        hAirblastDetector.SetMoveType(0, 0);
        hAirblastDetector.DispatchSpawn();
        hAirblastDetector.SetModelSimple("models/empty.mdl");
        hAirblastDetector.SetTeam(hOwner.GetTeam());
        hAirblastDetector.AcceptInput("SetParent", "!activator", hPhysProp, hPhysProp);

        AddTimer(-1, TickMoneyBagProjectile);
    }

    function CalcThrowVelocity()
    {
        return hOwner.EyeAngles().Forward() * 4400 + Vector(0, 0, 200);
    }

    function TickMoneyBagProjectile()
    {
        if (hAirblastDetector && GetPropInt(hAirblastDetector, "m_iDeflected") > 0)
        {
            OnDeflect(GetPropEntity(hAirblastDetector, "m_hDeflectOwner"));
            hAirblastDetector.Kill();
            hAirblastDetector = null;
        }

        local traceFloor = {
            start = hPhysProp.GetOrigin(),
            end = hPhysProp.GetOrigin() - Vector(0, 0, 25),
            ignore = hPhysProp
        };
        TraceLineEx(traceFloor);
        if (!safeget(traceFloor, "enthit", hOwner).IsPlayer())
        {
            hPhysProp.Kill();
            moneyBagFountainClass(hOwner, traceFloor.pos);
        }
    }

    function OnDeflect(deflectOwner)
    {
        printl("OnDeflect! "+deflectOwner);
    }
}